/**
 * Copyright 2019-2023 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of DesQ Shell (https://gitlab.com/DesQ/Shell)
 * DesQ Shell is the main Shell UI of the DesQ Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

// Local Headers
#include "Global.hpp"
#include "Logout.hpp"

#include <QScreen>
#include <signal.h>

// Wayland Headers
#include <wayqt/WayQtUtils.hpp>
#include <wayqt/Registry.hpp>
#include <wayqt/LayerShell.hpp>

#include <DFXdg.hpp>
#include <DFUtils.hpp>
#include <DFApplication.hpp>

WQt::Registry *wlRegistry;

PowerDialog *setupPowerDialog( QScreen *screen ) {
    PowerDialog *pwrDlg = new PowerDialog( screen );

    pwrDlg->showMaximized();

    if ( WQt::Utils::isWayland() ) {
        /** wl_output corresponding to @screen */
        wl_output *output = WQt::Utils::wlOutputFromQScreen( screen );

        WQt::LayerSurface *cls = wlRegistry->layerShell()->getLayerSurface(
            pwrDlg->windowHandle(),    // Window Handle
            output,                    // wl_output object - for multi-monitor support
            WQt::LayerShell::Overlay,  // Background layer
            "de-widget"                // Dummy namespace
        );

        /** Anchor everywhere */
        cls->setAnchors(
            WQt::LayerSurface::Top |
            WQt::LayerSurface::Bottom |
            WQt::LayerSurface::Left |
            WQt::LayerSurface::Right
        );

        /** Size - when */
        cls->setSurfaceSize( pwrDlg->size() );

        /** Nothing should move this */
        cls->setExclusiveZone( -1 );

        /** No keyboard interaction */
        cls->setKeyboardInteractivity( WQt::LayerSurface::Exclusive );

        /** Apply the changes */
        cls->apply();
    }

    return pwrDlg;
}


int main( int argc, char **argv ) {
    QDir cache( DFL::XDG::xdgCacheHome() );

    DFL::log = fopen( cache.filePath( "DesQ/Cask.log" ).toLocal8Bit().data(), "a" );

    qInstallMessageHandler( DFL::Logger );

    QByteArray date = QDateTime::currentDateTime().toString( "yyyyMMddThhmmss" ).toUtf8();

    qDebug() << "------------------------------------------------------------------------";
    qDebug() << "DesQ Logout started at" << date.constData();
    qDebug() << "------------------------------------------------------------------------\n";

    qputenv( "QT_WAYLAND_USE_BYPASSWINDOWMANAGERHINT", QByteArrayLiteral( "1" ) );

    DFL::Application *app = new DFL::Application( argc, argv );

    app->setOrganizationName( "DesQ" );
    app->setApplicationName( "Logout" );
    app->setApplicationVersion( PROJECT_VERSION );

    app->interceptSignal( SIGINT,  true );
    app->interceptSignal( SIGTERM, true );
    app->interceptSignal( SIGQUIT, true );
    app->interceptSignal( SIGSEGV, true );
    app->interceptSignal( SIGABRT, true );

    if ( app->lockApplication() ) {
        wlRegistry = new WQt::Registry( WQt::Wayland::display() );

        wlRegistry->setup();

        qDebug() << "Starting PowerDialog";

        for ( QScreen *screen: qApp->screens() ) {
            PowerDialog *dlg = setupPowerDialog( screen );

            QObject::connect(
                app, &DFL::Application::screenRemoved, [ = ] ( QScreen *scrn ) {
                    if ( scrn == screen ) {
                        dlg->close();
                        delete dlg;
                    }
                }
            );
        }

        QObject::connect(
            app, &QApplication::screenAdded, [ app ] ( QScreen *screen ) {
                PowerDialog *pwrDlg = setupPowerDialog( screen );
                QObject::connect(
                    app, &DFL::Application::screenRemoved, [ = ] ( QScreen *scrn ) {
                        if ( scrn == screen ) {
                            pwrDlg->close();
                            delete pwrDlg;
                        }
                    }
                );
            }
        );

        return app->exec();
    }

    return 0;
}
